@@ -3,7 +3,7 @@
 #
 # Basic example of parsing a file and dumping its parsed AST.
 #
-# Eli Bendersky [http://eli.thegreenplace.net]
+# Eli Bendersky [https://eli.thegreenplace.net/]
 # License: BSD
 #-----------------------------------------------------------------
 from __future__ import print_function
@@ -19,7 +19,9 @@ from pycparser import c_parser, c_ast, parse_file
 if __name__ == "__main__":
     argparser = argparse.ArgumentParser('Dump AST')
     argparser.add_argument('filename', help='name of file to parse')
+    argparser.add_argument('--coord', help='show coordinates in the dump',
+                           action='store_true')
     args = argparser.parse_args()
 
     ast = parse_file(args.filename, use_cpp=False)
-    ast.show()
+    ast.show(showcoord=args.coord)
